# Copyright (c) 2004, 2005 Martin Ankerl
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice, this list 
#      of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice, this list 
#      of conditions and the following disclaimer in the documentation and/or other materials 
#      provided with the distribution.
#    * Neither the name of Martin Ankerl nor the names of its contributors may be used to 
#      endorse or promote products derived from this software without specific prior written 
#      permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The Fox tabs do not look very nice. This simple handler uses buttons to show/hide frames to 
# emulate a similar, better looking widget.
class Custom_Tabs

	# Creates a new hander. btn_to_frame needs to be a hash where each key is
	# a button and each value a frame associated to the button.
	def initialize(btn_to_frame)
		@btn_to_frame = btn_to_frame
		create_tabs
	end	

	# Show the frame associated with the button show_btn.
	def show(show_btn)
		# disable everything
		@btn_to_frame.each_value do |frame|
			frame.hide
		end
		@btn_to_frame.each_key do |btn|
			btn.state = STATE_DOWN
		end
		# enable current tab
		@btn_to_frame[show_btn].show
		@btn_to_frame[show_btn].parent.recalc
		show_btn.state = STATE_UP		
	end	

	private

	# connect buttons
	def create_tabs
		@btn_to_frame.each do |btn, frame|
			btn.connect(SEL_COMMAND) { show(btn) }
			btn.connect(SEL_LEFTBUTTONPRESS) { show(btn) }
		end
	end
end
