# Copyright (c) 2004, 2005 Martin Ankerl
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice, this list 
#      of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice, this list 
#      of conditions and the following disclaimer in the documentation and/or other materials 
#      provided with the distribution.
#    * Neither the name of Martin Ankerl nor the names of its contributors may be used to 
#      endorse or promote products derived from this software without specific prior written 
#      permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


# Converts a Recursive_Open_Struct that contains filenames of PNG-icons into real icons.
class Icon_Loader
	# Create a new Icon_Loader. You need to specify the Fox-application.
	def initialize(app)
		@app = app
	end
	
	# Takes each attribute of the given Recursive_Open_Struct,
	# converts it into a real icon, and sets it.
	def cfg_to_icons(cfg)
		cfg.attrs.each do |attr|
			value = cfg.send(attr.to_sym)
			if (value.class == Recursive_Open_Struct)
				cfg_to_icons(value)
			else
				# value is a filename
				icon = make_icon(value)
				cfg.send((attr + "=").to_sym, icon)
			end
		end
	end
	
	# Constructs an icon from the given filename (from the icons directory).
	def make_icon(filename)
		filename = File.join("icons", filename)
		icon = nil
		File.open(filename, "rb") do |f|
			icon = FXPNGIcon.new(@app, f.read, 0)
		end
		icon.create
		icon
	end
end
