# Copyright (c) 2004, Christoph Heindl
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice, this list 
#      of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice, this list 
#      of conditions and the following disclaimer in the documentation and/or other materials 
#      provided with the distribution.
#    * Neither the name of Christoph Heindl nor the names of its contributors may be used to 
#      endorse or promote products derived from this software without specific prior written 
#      permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


require 'src/Utilities/Events.rb'
require 'thread'

# Provides a not-so-accurate timer class
class Timer
	include Event_Publisher

	# -- Connectable events
	
	# Periodically, when the specified timeout_secs have passed. Args: Timestamp
	ON_TIMER = Event_Publisher.next_global_eventID

	# Initializes and starts the timer with the given timeout attribute in seconds
	# This can also be a float value. I.e. timeout_secs == 0.5 triggers the timer
	# each 0.5 seconds
	def initialize(timeout_secs)
		self.init_events
		@mutex = Mutex.new
		@stop = false
		@thread = Thread.new(self, timeout_secs) do |caller, timeout|
			while (!caller.stop?)
				sleep(timeout)
				caller.fire_event(ON_TIMER, Time.now)
			end
		end
	end
	
	# Stop the timer
	def stop!
		@mutex.synchronize do
			@stop = true
		end
	end
	
	# Is the timer about to stop. or has already stopped?
	def stop?
		@mutex.synchronize do
			@stop
		end
	end
	
	# Wait for the timer to be killed. I.e. someone issues a stop!
	def join
		@thread.join
	end
end
